@@ -1,4 +1,31 @@
-/* SPDX-License-Identifier: MPL-2.0 */
+/*
+    Copyright (c) 2007-2016 Contributors as noted in the AUTHORS file
+
+    This file is part of libzmq, the ZeroMQ core engine in C++.
+
+    libzmq is free software; you can redistribute it and/or modify it under
+    the terms of the GNU Lesser General Public License (LGPL) as published
+    by the Free Software Foundation; either version 3 of the License, or
+    (at your option) any later version.
+
+    As a special exception, the Contributors give you permission to link
+    this library with independent modules to produce an executable,
+    regardless of the license terms of these independent modules, and to
+    copy and distribute the resulting executable under terms of your choice,
+    provided that you also meet, for each linked independent module, the
+    terms and conditions of the license of that module. An independent
+    module is a module which is not derived from or based on this library.
+    If you modify this library, you must extend this exception to your
+    version of the library.
+
+    libzmq is distributed in the hope that it will be useful, but WITHOUT
+    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
+    License for more details.
+
+    You should have received a copy of the GNU Lesser General Public License
+    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+*/
 
 #include "precompiled.hpp"
 
@@ -29,6 +56,7 @@
     do {                                                                       \
         delete poller_all;                                                     \
         delete poller_in;                                                      \
+        delete poller_control;                                                 \
         delete poller_receive_blocked;                                         \
         delete poller_send_blocked;                                            \
         delete poller_both_blocked;                                            \
@@ -47,6 +75,20 @@
 
 #endif //  ZMQ_HAVE_POLLER
 
+
+// Control socket messages
+
+typedef struct
+{
+    uint64_t msg_in;
+    uint64_t bytes_in;
+    uint64_t msg_out;
+    uint64_t bytes_out;
+} zmq_socket_stats_t;
+
+
+// Utility functions
+
 static int
 capture (class zmq::socket_base_t *capture_, zmq::msg_t *msg_, int more_ = 0)
 {
@@ -67,7 +109,9 @@ capture (class zmq::socket_base_t *capture_, zmq::msg_t *msg_, int more_ = 0)
 }
 
 static int forward (class zmq::socket_base_t *from_,
+                    zmq_socket_stats_t *from_stats_,
                     class zmq::socket_base_t *to_,
+                    zmq_socket_stats_t *to_stats_,
                     class zmq::socket_base_t *capture_,
                     zmq::msg_t *msg_)
 {
@@ -75,6 +119,7 @@ static int forward (class zmq::socket_base_t *from_,
     for (unsigned int i = 0; i < zmq::proxy_burst_size; i++) {
         int more;
         size_t moresz;
+        size_t complete_msg_size = 0;
 
         // Forward all the parts of one message
         while (true) {
@@ -86,6 +131,8 @@ static int forward (class zmq::socket_base_t *from_,
                 return -1;
             }
 
+            complete_msg_size += msg_->size ();
+
             moresz = sizeof more;
             rc = from_->getsockopt (ZMQ_RCVMORE, &more, &moresz);
             if (unlikely (rc < 0))
@@ -103,15 +150,76 @@ static int forward (class zmq::socket_base_t *from_,
             if (more == 0)
                 break;
         }
+
+        // A multipart message counts as 1 packet:
+        from_stats_->msg_in++;
+        from_stats_->bytes_in += complete_msg_size;
+        to_stats_->msg_out++;
+        to_stats_->bytes_out += complete_msg_size;
     }
 
     return 0;
 }
 
+static int loop_and_send_multipart_stat (zmq::socket_base_t *control_,
+                                         uint64_t stat_,
+                                         bool first_,
+                                         bool more_)
+{
+    int rc;
+    zmq::msg_t msg;
+
+    //  VSM of 8 bytes can't fail to init
+    msg.init_size (sizeof (uint64_t));
+    memcpy (msg.data (), &stat_, sizeof (uint64_t));
+
+    //  if the first message is handed to the pipe successfully then the HWM
+    //  is not full, which means failures are due to interrupts (on Windows pipes
+    //  are TCP sockets), so keep retrying
+    do {
+        rc = control_->send (&msg, more_ ? ZMQ_SNDMORE : 0);
+    } while (!first_ && rc != 0 && errno == EAGAIN);
+
+    return rc;
+}
+
+static int reply_stats (zmq::socket_base_t *control_,
+                        const zmq_socket_stats_t *frontend_stats_,
+                        const zmq_socket_stats_t *backend_stats_)
+{
+    // first part: frontend stats - the first send might fail due to HWM
+    if (loop_and_send_multipart_stat (control_, frontend_stats_->msg_in, true,
+                                      true)
+        != 0)
+        return -1;
+
+    loop_and_send_multipart_stat (control_, frontend_stats_->bytes_in, false,
+                                  true);
+    loop_and_send_multipart_stat (control_, frontend_stats_->msg_out, false,
+                                  true);
+    loop_and_send_multipart_stat (control_, frontend_stats_->bytes_out, false,
+                                  true);
+
+    // second part: backend stats
+    loop_and_send_multipart_stat (control_, backend_stats_->msg_in, false,
+                                  true);
+    loop_and_send_multipart_stat (control_, backend_stats_->bytes_in, false,
+                                  true);
+    loop_and_send_multipart_stat (control_, backend_stats_->msg_out, false,
+                                  true);
+    loop_and_send_multipart_stat (control_, backend_stats_->bytes_out, false,
+                                  false);
+
+    return 0;
+}
+
+
 #ifdef ZMQ_HAVE_POLLER
+
 int zmq::proxy (class socket_base_t *frontend_,
                 class socket_base_t *backend_,
-                class socket_base_t *capture_)
+                class socket_base_t *capture_,
+                class socket_base_t *control_)
 {
     msg_t msg;
     int rc = msg.init ();
@@ -121,6 +229,9 @@ int zmq::proxy (class socket_base_t *frontend_,
     //  The algorithm below assumes ratio of requests and replies processed
     //  under full load to be 1:1.
 
+    int more;
+    size_t moresz = sizeof (more);
+
     //  Proxy can be in these three states
     enum
     {
@@ -134,7 +245,12 @@ int zmq::proxy (class socket_base_t *frontend_,
     bool frontend_out = false;
     bool backend_in = false;
     bool backend_out = false;
+    bool control_in = false;
     zmq::socket_poller_t::event_t events[3];
+    zmq_socket_stats_t frontend_stats;
+    zmq_socket_stats_t backend_stats;
+    memset (&frontend_stats, 0, sizeof (frontend_stats));
+    memset (&backend_stats, 0, sizeof (backend_stats));
 
     //  Don't allocate these pollers from stack because they will take more than 900 kB of stack!
     //  On Windows this blows up default stack of 1 MB and aborts the program.
@@ -143,6 +259,8 @@ int zmq::proxy (class socket_base_t *frontend_,
       new (std::nothrow) zmq::socket_poller_t; //  Poll for everything.
     zmq::socket_poller_t *poller_in = new (std::nothrow) zmq::
       socket_poller_t; //  Poll only 'ZMQ_POLLIN' on all sockets. Initial blocking poll in loop.
+    zmq::socket_poller_t *poller_control = new (std::nothrow) zmq::
+      socket_poller_t; //  Poll only for 'ZMQ_POLLIN' on 'control_', when proxy is paused.
     zmq::socket_poller_t *poller_receive_blocked = new (std::nothrow)
       zmq::socket_poller_t; //  All except 'ZMQ_POLLIN' on 'frontend_'.
 
@@ -172,7 +290,7 @@ int zmq::proxy (class socket_base_t *frontend_,
     } else
         frontend_equal_to_backend = true;
 
-    if (poller_all == NULL || poller_in == NULL
+    if (poller_all == NULL || poller_in == NULL || poller_control == NULL
         || poller_receive_blocked == NULL
         || ((poller_send_blocked == NULL || poller_both_blocked == NULL)
             && !frontend_equal_to_backend)) {
@@ -232,6 +350,30 @@ int zmq::proxy (class socket_base_t *frontend_,
         CHECK_RC_EXIT_ON_FAILURE ();
     }
 
+    //  Register 'control_' with pollers.
+    if (control_ != NULL) {
+        rc = poller_all->add (control_, NULL, ZMQ_POLLIN);
+        CHECK_RC_EXIT_ON_FAILURE ();
+        rc = poller_in->add (control_, NULL, ZMQ_POLLIN);
+        CHECK_RC_EXIT_ON_FAILURE ();
+        rc = poller_control->add (
+          control_, NULL,
+          ZMQ_POLLIN); //  When proxy is paused we wait only for ZMQ_POLLIN on 'control_' socket.
+        CHECK_RC_EXIT_ON_FAILURE ();
+        rc = poller_receive_blocked->add (control_, NULL, ZMQ_POLLIN);
+        CHECK_RC_EXIT_ON_FAILURE ();
+        if (!frontend_equal_to_backend) {
+            rc = poller_send_blocked->add (control_, NULL, ZMQ_POLLIN);
+            CHECK_RC_EXIT_ON_FAILURE ();
+            rc = poller_both_blocked->add (control_, NULL, ZMQ_POLLIN);
+            CHECK_RC_EXIT_ON_FAILURE ();
+            rc = poller_frontend_only->add (control_, NULL, ZMQ_POLLIN);
+            CHECK_RC_EXIT_ON_FAILURE ();
+            rc = poller_backend_only->add (control_, NULL, ZMQ_POLLIN);
+            CHECK_RC_EXIT_ON_FAILURE ();
+        }
+    }
+
     bool request_processed, reply_processed;
 
     while (state != terminated) {
@@ -260,14 +402,58 @@ int zmq::proxy (class socket_base_t *frontend_,
                 if (events[i].socket == backend_) {
                     backend_in = (events[i].events & ZMQ_POLLIN) != 0;
                     backend_out = (events[i].events & ZMQ_POLLOUT) != 0;
+                } else if (events[i].socket == control_)
+                    control_in = (events[i].events & ZMQ_POLLIN) != 0;
+        }
+
+
+        //  Process a control command if any.
+        if (control_in) {
+            rc = control_->recv (&msg, 0);
+            CHECK_RC_EXIT_ON_FAILURE ();
+            rc = control_->getsockopt (ZMQ_RCVMORE, &more, &moresz);
+            if (unlikely (rc < 0) || more) {
+                PROXY_CLEANUP ();
+                return close_and_return (&msg, -1);
+            }
+
+            //  Copy message to capture socket if any.
+            rc = capture (capture_, &msg);
+            CHECK_RC_EXIT_ON_FAILURE ();
+
+            if (msg.size () == 5 && memcmp (msg.data (), "PAUSE", 5) == 0) {
+                state = paused;
+                poller_wait = poller_control;
+            } else if (msg.size () == 6
+                       && memcmp (msg.data (), "RESUME", 6) == 0) {
+                state = active;
+                poller_wait = poller_in;
+            } else {
+                if (msg.size () == 9
+                    && memcmp (msg.data (), "TERMINATE", 9) == 0)
+                    state = terminated;
+                else {
+                    if (msg.size () == 10
+                        && memcmp (msg.data (), "STATISTICS", 10) == 0) {
+                        rc = reply_stats (control_, &frontend_stats,
+                                          &backend_stats);
+                        CHECK_RC_EXIT_ON_FAILURE ();
+                    } else {
+                        //  This is an API error, we assert
+                        puts ("E: invalid command sent to proxy");
+                        zmq_assert (false);
+                    }
                 }
+            }
+            control_in = false;
         }
 
         if (state == active) {
             //  Process a request, 'ZMQ_POLLIN' on 'frontend_' and 'ZMQ_POLLOUT' on 'backend_'.
             //  In case of frontend_==backend_ there's no 'ZMQ_POLLOUT' event.
             if (frontend_in && (backend_out || frontend_equal_to_backend)) {
-                rc = forward (frontend_, backend_, capture_, &msg);
+                rc = forward (frontend_, &frontend_stats, backend_,
+                              &backend_stats, capture_, &msg);
                 CHECK_RC_EXIT_ON_FAILURE ();
                 request_processed = true;
                 frontend_in = backend_out = false;
@@ -279,7 +465,8 @@ int zmq::proxy (class socket_base_t *frontend_,
             //  covers all of the cases. 'backend_in' is always false if frontend_==backend_ due to
             //  design in 'for' event processing loop.
             if (backend_in && frontend_out) {
-                rc = forward (backend_, frontend_, capture_, &msg);
+                rc = forward (backend_, &backend_stats, frontend_,
+                              &frontend_stats, capture_, &msg);
                 CHECK_RC_EXIT_ON_FAILURE ();
                 reply_processed = true;
                 backend_in = frontend_out = false;
@@ -348,7 +535,8 @@ int zmq::proxy (class socket_base_t *frontend_,
 
 int zmq::proxy (class socket_base_t *frontend_,
                 class socket_base_t *backend_,
-                class socket_base_t *capture_)
+                class socket_base_t *capture_,
+                class socket_base_t *control_)
 {
     msg_t msg;
     int rc = msg.init ();
@@ -358,12 +546,20 @@ int zmq::proxy (class socket_base_t *frontend_,
     //  The algorithm below assumes ratio of requests and replies processed
     //  under full load to be 1:1.
 
+    int more;
+    size_t moresz;
     zmq_pollitem_t items[] = {{frontend_, 0, ZMQ_POLLIN, 0},
-                              {backend_, 0, ZMQ_POLLIN, 0}};
-    int qt_poll_items = 2;
+                              {backend_, 0, ZMQ_POLLIN, 0},
+                              {control_, 0, ZMQ_POLLIN, 0}};
+    int qt_poll_items = (control_ ? 3 : 2);
     zmq_pollitem_t itemsout[] = {{frontend_, 0, ZMQ_POLLOUT, 0},
                                  {backend_, 0, ZMQ_POLLOUT, 0}};
 
+    zmq_socket_stats_t frontend_stats;
+    memset (&frontend_stats, 0, sizeof (frontend_stats));
+    zmq_socket_stats_t backend_stats;
+    memset (&backend_stats, 0, sizeof (backend_stats));
+
     //  Proxy can be in these three states
     enum
     {
@@ -388,9 +584,48 @@ int zmq::proxy (class socket_base_t *frontend_,
             }
         }
 
+        //  Process a control command if any
+        if (control_ && items[2].revents & ZMQ_POLLIN) {
+            rc = control_->recv (&msg, 0);
+            if (unlikely (rc < 0))
+                return close_and_return (&msg, -1);
+
+            moresz = sizeof more;
+            rc = control_->getsockopt (ZMQ_RCVMORE, &more, &moresz);
+            if (unlikely (rc < 0) || more)
+                return close_and_return (&msg, -1);
+
+            //  Copy message to capture socket if any
+            rc = capture (capture_, &msg);
+            if (unlikely (rc < 0))
+                return close_and_return (&msg, -1);
+
+            if (msg.size () == 5 && memcmp (msg.data (), "PAUSE", 5) == 0)
+                state = paused;
+            else if (msg.size () == 6 && memcmp (msg.data (), "RESUME", 6) == 0)
+                state = active;
+            else if (msg.size () == 9
+                     && memcmp (msg.data (), "TERMINATE", 9) == 0)
+                state = terminated;
+            else {
+                if (msg.size () == 10
+                    && memcmp (msg.data (), "STATISTICS", 10) == 0) {
+                    rc =
+                      reply_stats (control_, &frontend_stats, &backend_stats);
+                    if (unlikely (rc < 0))
+                        return close_and_return (&msg, -1);
+                } else {
+                    //  This is an API error, we assert
+                    puts ("E: invalid command sent to proxy");
+                    zmq_assert (false);
+                }
+            }
+        }
+        //  Process a request
         if (state == active && items[0].revents & ZMQ_POLLIN
             && (frontend_ == backend_ || itemsout[1].revents & ZMQ_POLLOUT)) {
-            rc = forward (frontend_, backend_, capture_, &msg);
+            rc = forward (frontend_, &frontend_stats, backend_, &backend_stats,
+                          capture_, &msg);
             if (unlikely (rc < 0))
                 return close_and_return (&msg, -1);
         }
@@ -398,7 +633,8 @@ int zmq::proxy (class socket_base_t *frontend_,
         if (state == active && frontend_ != backend_
             && items[1].revents & ZMQ_POLLIN
             && itemsout[0].revents & ZMQ_POLLOUT) {
-            rc = forward (backend_, frontend_, capture_, &msg);
+            rc = forward (backend_, &backend_stats, frontend_, &frontend_stats,
+                          capture_, &msg);
             if (unlikely (rc < 0))
                 return close_and_return (&msg, -1);
         }
