@@ -1,4 +1,31 @@
-/* SPDX-License-Identifier: MPL-2.0 */
+/*
+    Copyright (c) 2007-2016 Contributors as noted in the AUTHORS file
+
+    This file is part of libzmq, the ZeroMQ core engine in C++.
+
+    libzmq is free software; you can redistribute it and/or modify it under
+    the terms of the GNU Lesser General Public License (LGPL) as published
+    by the Free Software Foundation; either version 3 of the License, or
+    (at your option) any later version.
+
+    As a special exception, the Contributors give you permission to link
+    this library with independent modules to produce an executable,
+    regardless of the license terms of these independent modules, and to
+    copy and distribute the resulting executable under terms of your choice,
+    provided that you also meet, for each linked independent module, the
+    terms and conditions of the license of that module. An independent
+    module is a module which is not derived from or based on this library.
+    If you modify this library, you must extend this exception to your
+    version of the library.
+
+    libzmq is distributed in the hope that it will be useful, but WITHOUT
+    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
+    License for more details.
+
+    You should have received a copy of the GNU Lesser General Public License
+    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+*/
 
 // "Tell them I was a writer.
 //  A maker of software.
@@ -1743,12 +1770,10 @@ int zmq_proxy_steerable (void *frontend_,
         errno = EFAULT;
         return -1;
     }
-#ifdef ZMQ_HAVE_WINDOWS
-    errno = WSAEOPNOTSUPP;
-#else
-    errno = EOPNOTSUPP;
-#endif
-    return -1;
+    return zmq::proxy (static_cast<zmq::socket_base_t *> (frontend_),
+                       static_cast<zmq::socket_base_t *> (backend_),
+                       static_cast<zmq::socket_base_t *> (capture_),
+                       static_cast<zmq::socket_base_t *> (control_));
 }
 
 //  The deprecated device functionality
